/*
 * Autor: Andrew Solera (asolera@prefeitura.sp.gov.br)
 * Data: 24/05/2021
 * Descrição: Processa os dados brutos para as tabelas normalizadas.
 */

-- Habilita a extensão do "unaccent" para remover acentos na pesquisa textual
CREATE EXTENSION unaccent;

-- Índices para pesquisa textual
CREATE INDEX ix_municipe_nome ON tb_municipe USING GIN (nome_vector);
CREATE INDEX ix_contribuicao_integra ON tb_contribuicao USING GIN (conteudo_vector);
CREATE INDEX ix_subtema_nome ON tb_subtema USING GIN (nome_vector);
CREATE INDEX ix_subtema_devolutiva ON tb_subtema USING GIN (devolutiva_vector);
-- Munícipes
INSERT INTO tb_municipe (id_origem, nome, nome_vector)
SELECT id_municipe, nome, to_tsvector('portuguese', unaccent(nome)) FROM raw_municipe ORDER BY id_municipe ASC;

-- Canal de participação
INSERT INTO tb_canal (nome, introducao) SELECT DISTINCT nome_canal, introducao FROM raw_canal ORDER BY nome_canal ASC;

-- Contribuições
INSERT INTO tb_contribuicao (id_origem, id_canal, id_municipe, conteudo, conteudo_vector)
SELECT      root.id_contribuicao,
            c.id,
            m.id,
            root.contribuicao_integra,
            to_tsvector('portuguese', unaccent(root.contribuicao_integra))
FROM        raw_contribuicao AS root
INNER JOIN  tb_canal AS c ON c.nome = root.canal
INNER JOIN  tb_municipe AS m ON m.id_origem = root.id_municipe
ORDER BY    root.id_contribuicao ASC;

-- Subprefeituras
INSERT INTO tb_subprefeitura (nome) SELECT DISTINCT subprefeitura FROM raw_sugestao ORDER BY subprefeitura ASC;

-- Categorias
INSERT INTO tb_categoria (nome) SELECT DISTINCT categoria FROM raw_subtema ORDER BY categoria ASC;

-- Eixos
INSERT INTO tb_eixo (nome) SELECT DISTINCT eixo FROM raw_subtema ORDER BY eixo ASC;

-- Secretarias
INSERT INTO tb_secretaria (sigla) SELECT DISTINCT secretaria_sigla FROM raw_subtema ORDER BY secretaria_sigla ASC;

-- Subtemas
INSERT INTO tb_subtema (id_origem, id_categoria, id_eixo, id_secretaria, nome, nome_vector, devolutiva, devolutiva_vector)
SELECT      root.id_subtema,
            c.id,
            e.id,
            s.id,
            root.subtema,
            to_tsvector('portuguese', unaccent(root.subtema)),
            root.devolutiva,
            to_tsvector('portuguese', unaccent(root.devolutiva))
FROM        raw_subtema root
INNER JOIN  tb_categoria AS c ON c.nome = root.categoria
INNER JOIN  tb_eixo AS e ON e.nome = root.eixo
INNER JOIN  tb_secretaria AS s ON s.sigla = root.secretaria_sigla
ORDER BY    root.id_subtema ASC;

-- Sugestões
INSERT INTO tb_sugestao (id_origem, id_contribuicao, id_subprefeitura, id_subtema, conteudo)
SELECT      root.id_sugestao,
            c.id,
            s.id,
            st.id,
            root.sugestao
FROM        raw_sugestao AS root
INNER JOIN  tb_contribuicao AS c ON c.id_origem = root.id_contribuicao
INNER JOIN  tb_subprefeitura AS s ON s.nome = root.subprefeitura
INNER JOIN  tb_subtema AS st ON st.id_origem = root.id_subtema
ORDER BY    root.id_sugestao ASC;

-- View utilizada na pesquisa de devolutivas
CREATE VIEW vw_subtemas_pesquisa AS
SELECT      con.id AS contribuicao_id,
            mun.id AS municipe_id,
            mun.nome AS municipe_nome,
            mun.nome_vector AS municipe_nome_vector,
            can.id AS canal_id,
            can.nome AS canal_nome,
            can.introducao AS canal_introducao,
            con.conteudo AS contribuicao_integra,
            con.conteudo_vector AS contribuicao_integra_vector,
            sug.id AS sugestao_id,
            sug.conteudo AS sugestao_conteudo,
            sub.id AS subprefeitura_id,
            sub.nome AS subprefeitura_nome,
            sbt.id AS subtema_id,
            sbt.nome AS subtema_nome,
            sbt.nome_vector AS subtema_nome_vector,
            cat.id AS categoria_id,
            cat.nome AS categoria_nome,
            eix.id AS eixo_id,
            eix.nome AS eixo_nome,
            sec.id AS secretaria_id,
            sec.sigla AS secretaria_sigla,
            sbt.devolutiva AS subtema_devolutiva,
            sbt.devolutiva_vector AS subtema_devolutiva_vector
FROM        tb_contribuicao AS con
INNER JOIN  tb_municipe AS mun ON mun.id = con.id_municipe
INNER JOIN  tb_sugestao AS sug ON sug.id_contribuicao = con.id
INNER JOIN  tb_subtema AS sbt ON sbt.id = sug.id_subtema
INNER JOIN  tb_canal AS can ON can.id = con.id_canal
INNER JOIN  tb_subprefeitura AS sub ON sub.id = sug.id_subprefeitura
INNER JOIN  tb_categoria AS cat ON cat.id = sbt.id_categoria
INNER JOIN  tb_eixo AS eix ON eix.id = sbt.id_eixo
INNER JOIN  tb_secretaria AS sec ON sec.id = sbt.id_secretaria
ORDER BY    con.id, sug.id ASC;